﻿using System;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace LPAutoScroll1 {
	public partial class LPAutoScroll1 : Form {
		private System.Windows.Forms.Panel topPanel;
		private System.Windows.Forms.ListBox lb;
		private int clickX;
		private Panel bottomPanel;
		private Panel innerPanel;		// coordinates of the logical click point
		private int clickY;

		public LPAutoScroll1() {
			InitializeComponent();
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			Application.EnableVisualStyles();
			Application.SetCompatibleTextRenderingDefault(false);
			Application.Run(new LPAutoScroll1());
		}

		private void InitializeComponent() {
			this.topPanel = new System.Windows.Forms.Panel();
			this.lb = new System.Windows.Forms.ListBox();
			this.bottomPanel = new System.Windows.Forms.Panel();
			this.innerPanel = new System.Windows.Forms.Panel();
			this.bottomPanel.SuspendLayout();
			this.SuspendLayout();
			// 
			// topPanel
			// 
			this.topPanel.AutoScroll = true;
			this.topPanel.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(255)))), ((int)(((byte)(235)))), ((int)(((byte)(235)))));
			this.topPanel.Location = new System.Drawing.Point(20, 20);
			this.topPanel.Name = "topPanel";
			this.topPanel.Size = new System.Drawing.Size(300, 200);
			this.topPanel.TabIndex = 0;
			this.topPanel.MouseMove += new System.Windows.Forms.MouseEventHandler(this.topPanel_MouseMove);
			this.topPanel.MouseDown += new System.Windows.Forms.MouseEventHandler(this.topPanel_MouseDown);
			this.topPanel.MouseUp += new System.Windows.Forms.MouseEventHandler(this.topPanel_MouseUp);
			// 
			// lb
			// 
			this.lb.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
            | System.Windows.Forms.AnchorStyles.Right)));
			this.lb.Font = new System.Drawing.Font("Courier New", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
			this.lb.FormattingEnabled = true;
			this.lb.ItemHeight = 16;
			this.lb.Location = new System.Drawing.Point(349, 21);
			this.lb.Name = "lb";
			this.lb.Size = new System.Drawing.Size(525, 420);
			this.lb.TabIndex = 1;
			// 
			// bottomPanel
			// 
			this.bottomPanel.AutoScroll = true;
			this.bottomPanel.BackColor = System.Drawing.Color.White;
			this.bottomPanel.Controls.Add(this.innerPanel);
			this.bottomPanel.Location = new System.Drawing.Point(20, 240);
			this.bottomPanel.Name = "bottomPanel";
			this.bottomPanel.Size = new System.Drawing.Size(300, 200);
			this.bottomPanel.TabIndex = 1;
			// 
			// innerPanel
			// 
			this.innerPanel.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(235)))), ((int)(((byte)(255)))), ((int)(((byte)(235)))));
			this.innerPanel.Location = new System.Drawing.Point(53, 40);
			this.innerPanel.Name = "innerPanel";
			this.innerPanel.Size = new System.Drawing.Size(119, 96);
			this.innerPanel.TabIndex = 0;
			this.innerPanel.MouseMove += new System.Windows.Forms.MouseEventHandler(this.innerPanel_MouseMove);
			this.innerPanel.MouseDown += new System.Windows.Forms.MouseEventHandler(this.innerPanel_MouseDown);
			this.innerPanel.MouseUp += new System.Windows.Forms.MouseEventHandler(this.innerPanel_MouseUp);
			// 
			// LPAutoScroll1
			// 
			this.AutoScaleDimensions = new System.Drawing.SizeF(6F, 13F);
			this.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Font;
			this.ClientSize = new System.Drawing.Size(894, 459);
			this.Controls.Add(this.bottomPanel);
			this.Controls.Add(this.lb);
			this.Controls.Add(this.topPanel);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedSingle;
			this.Name = "LPAutoScroll1";
			this.Text = "AutoScroll";
			this.Load += new System.EventHandler(this.Form1_Load);
			this.bottomPanel.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		private void log(string s) {
			lb.Items.Add(s);
			lb.TopIndex=lb.Items.Count-1;
		}

		private void logScrolls(string title) {
			log("Scrollbar values, "+title);
			log("     topPanel: min="+topPanel.HorizontalScroll.Minimum.ToString("D3")+" & "+
				topPanel.VerticalScroll.Minimum.ToString("D3")+
				"  max="+topPanel.HorizontalScroll.Maximum.ToString("D3")+" & "+
				topPanel.VerticalScroll.Maximum.ToString("D3")+
				"  val="+topPanel.HorizontalScroll.Value.ToString("D3")+" & "+
				topPanel.VerticalScroll.Value.ToString("D3"));
			log("  bottomPanel: min="+bottomPanel.HorizontalScroll.Minimum.ToString("D3")+" & "+
				bottomPanel.VerticalScroll.Minimum.ToString("D3")+
				"  max="+bottomPanel.HorizontalScroll.Maximum.ToString("D3")+" & "+
				bottomPanel.VerticalScroll.Maximum.ToString("D3")+
				"  val="+bottomPanel.HorizontalScroll.Value.ToString("D3")+" & "+
				bottomPanel.VerticalScroll.Value.ToString("D3"));
		}

		private void Form1_Load(object sender, EventArgs e) {
			log("Panel size = "+topPanel.Size);
			logScrolls("without population");
			addLabelsToPanel(topPanel);
			innerPanel.Bounds=new Rectangle(0, 0, 1000, 920);
			addLabelsToPanel(innerPanel);
			logScrolls("with population");
		}

		private void addLabelsToPanel(Panel panel) {
			// populate the Panel with a lot of Labels
			Label lbl=null;
			for (int i=0; i<30; i++) {
				lbl=new Label();
				lbl.Text="Label "+i;
				lbl.Bounds=new Rectangle(30*i, 30*i, 100, 20);
				lbl.BackColor=Color.Yellow;
				lbl.TextAlign=ContentAlignment.MiddleCenter;
				lbl.Enabled=false;
				panel.Controls.Add(lbl);
			}
			log("Last label: bounds="+lbl.Bounds);
			// lbl.Visible=false;	// optional proof invisible Controls don't increase the hull
		}

		private void topPanel_MouseDown(object sender, MouseEventArgs e) {
			clickX=topPanel.HorizontalScroll.Value+e.X;
			clickY=topPanel.VerticalScroll.Value+e.Y;
			if (e.Button==MouseButtons.Left) {
				Cursor=Cursors.Hand;
				log("Start topPanel panning at "+new Point(clickX, clickY));
			}
		}

		private void topPanel_MouseUp(object sender, MouseEventArgs e) {
			Cursor=Cursors.Default;
			logScrolls("topPanel mouse up");
		}

		private void topPanel_MouseMove(object sender, MouseEventArgs e) {
			if (e.Button==MouseButtons.Left) {
				setScrollValue(topPanel.HorizontalScroll, clickX-e.X);
				setScrollValue(topPanel.VerticalScroll, clickY-e.Y);
			}
		}

		private void setScrollValue(ScrollProperties scrollbar, int val) {
			if (val<scrollbar.Minimum) val=scrollbar.Minimum;
			if (val>scrollbar.Maximum) val=scrollbar.Maximum;
			scrollbar.Value=val;
		}

		private void innerPanel_MouseDown(object sender, MouseEventArgs e) {
			clickX=bottomPanel.HorizontalScroll.Value+MousePosition.X;
			clickY=bottomPanel.VerticalScroll.Value+MousePosition.Y;
			if (e.Button==MouseButtons.Left) {
				Cursor=Cursors.Hand;
				log("Start innerPanel panning at "+new Point(clickX, clickY));
			}
		}

		private void innerPanel_MouseUp(object sender, MouseEventArgs e) {
			Cursor=Cursors.Default;
			logScrolls("innerPanel mouse up");
		}

		private void innerPanel_MouseMove(object sender, MouseEventArgs e) {
			if (e.Button==MouseButtons.Left) {
				setScrollValue(bottomPanel.HorizontalScroll, clickX-MousePosition.X);
				setScrollValue(bottomPanel.VerticalScroll, clickY-MousePosition.Y);
			}
		}
	}
}
